package com.capitalware.ufmsm.models;

import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;

/**
 * This class provides a sort table model for a JTable.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class SortFilterModel extends AbstractTableModel
{
	private static final long serialVersionUID = 1L;
   private StatusQueueModel model;
   private int sortColumn;
   private Row[] rows;

   /**
    * The Constructor
    * @param m the table model
    */
	public SortFilterModel(StatusQueueModel m)
   {  
	   model = m;
      rows = new Row[model.getRowCount()];
      for (int i = 0; i < rows.length; i++)
      {  rows[i] = new Row();
         rows[i].index = i;
      }
   }

	public void updatedSortFilter()
	{
      rows = new Row[model.getRowCount()];
      for (int i = 0; i < rows.length; i++)
      {  rows[i] = new Row();
         rows[i].index = i;
      }
	}
	
   /**
    * Sort action
    * @param c column to sort the table on
    */
   public void sort(int c)
   {
      sortColumn = c;
      Arrays.sort(rows);
      fireTableDataChanged();
   }

   /**
    * Mouse listener to listen for double clicks to sort the table.
    * @param table
    */
   public void addMouseListener(final JTable table)
   {
      table.getTableHeader().addMouseListener(new MouseAdapter()
         {
            public void mouseClicked(MouseEvent event)
            {  // check for double click
               if (event.getClickCount() < 2) return;

               // find column of click and
               int tableColumn = table.columnAtPoint(event.getPoint());

               // translate to table model index and sort
               int modelColumn = table.convertColumnIndexToModel(tableColumn);
               sort(modelColumn);
            }
         });
   }

   /* compute the moved row for the three methods that access
      model elements
   */

   /**
    * Retrieve the value at a particular cell.
    * @param r row number
    * @param c column number
    */
   public Object getValueAt(int r, int c)
   {  
      return model.getValueAt(rows[r].index, c);
   }

   /**
    * Retrieve the data for an entire row.
    * @param r
    * @return objectArray
    */
   public Object[] getRowData(int r)
   {
      return model.getRowData(rows[r].index);
   }

   /**
    * Does the particular cell allow editing of its contents.
    * @param r row number
    * @param c column number
    * @return true if the cell is editable; otherwise false.
    */
   public boolean isCellEditable(int r, int c)
   {
      // return model.isCellEditable(rows[r].index, c);
      return false;
   }

   /**
    * Set the value at a particular cell.
    * @param r row number
    * @param c column number
    */
   public void setValueAt(Object aValue, int r, int c)
   {  model.setValueAt(aValue, rows[r].index, c);
   }

   /* delegate all remaining methods to the model
   */

   /**
    * Retrieve the number of rows in the table
    * @return the number of rows in the table
    */
   public int getRowCount()
   {  return model.getRowCount();
   }

   /**
    * Retrieve the number of columns in the table
    * @return the number of columns in the table
    */
   public int getColumnCount()
   {  return model.getColumnCount();
   }

   /**
    * Retrieve the name of the column.
    * @return the name of the column
    */
   public String getColumnName(int c)
   {  return model.getColumnName(c);
   }

   /**
    * Retrieve the column class for this particular column.
    * @return the class for this particular column
    */
   public Class getColumnClass(int c)
   {  return model.getColumnClass(c);
   }

   /**
    *  This inner class holds the index of the model row
    *  Rows are compared by looking at the model row entries
    *  in the sort column
    */
   private class Row implements Comparable
   {
      public int index;

      /**
       * Compare given object to current object at index.
       * @param Object row object
       * @return -1 for less than, 0 for the same or 1 for greater than
       */
      public int compareTo(Object other)
      {
         Row otherRow = (Row)other;
         Object a = model.getValueAt(index, sortColumn);
         Object b = model.getValueAt(otherRow.index, sortColumn);
         
         // Descending order
         if (a instanceof Comparable)
            return ((Comparable)b).compareTo(a);
         else
            return index - otherRow.index;
         
//         // Ascending order
//         if (a instanceof Comparable)
//            return ((Comparable)a).compareTo(b);
//         else
//            return index - otherRow.index;
      }
   }
}
