package com.capitalware.ufmsm.mq;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.security.InvalidAlgorithmParameterException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertStore;
import java.security.cert.LDAPCertStoreParameters;
import java.util.ArrayList;
import java.util.Hashtable;

import com.capitalware.ufmsm.mq.objects.MQObject;
import com.capitalware.ufmsm.mq.objects.QueueObject;

import com.ibm.mq.MQC;
import com.ibm.mq.MQException;
import com.ibm.mq.MQQueue;
import com.ibm.mq.MQQueueManager;
import com.ibm.mq.MQSecurityExit;

/**
 * A base class to define MQ connection and disconnection methods.
 *
 * @author Roger Lacroix, Capitalware Inc.
 * @version 1.0
 * @license Apache 2 License
 */
public class MyMQ extends Thread
{
   protected   MQObject        mqo;
   protected   MQQueueManager  _qMgr = null;

   /**
    * The constructor
    * @param logger
    */
   public MyMQ(MQObject mqo)
   {
      super();
      this.mqo = mqo;
   }

   /**
    * Connect to MQ Server
    *
    * @throws MQException
    * @throws MalformedURLException
    * @throws ClassNotFoundException
    * @throws IllegalAccessException
    * @throws InstantiationException
    * @throws NoSuchAlgorithmException
    * @throws InvalidAlgorithmParameterException
    */
   public void connect() throws MQException, MalformedURLException, ClassNotFoundException, InstantiationException, IllegalAccessException, InvalidAlgorithmParameterException, NoSuchAlgorithmException
   {
      MQException.log = null;
      boolean setUserIDPassword = true;

      if (mqo.channelName != null)
      {
         Hashtable mqht = new Hashtable();

         mqht.put(MQC.CHANNEL_PROPERTY, mqo.channelName);
         mqht.put(MQC.HOST_NAME_PROPERTY, mqo.hostname);
         mqht.put(MQC.PORT_PROPERTY, new Integer(mqo.port));


         if (mqo.securityExit != null)
         {
            if ( ("biz.capitalware.mqausx.MQAUSXJ".equals(mqo.securityExit)) ||
                 ("biz.capitalware.mqausx.MQAUSXJ2EE".equals(mqo.securityExit)) )
            {
               mqht.put(MQC.SECURITY_EXIT_PROPERTY, CWSX.getMQAUSXJ(mqo));
               
               /* Did we already pass the info to MQAUSXJ or MQAUSXJ2EE? */
               if (mqo.securityExitData != null)
                  setUserIDPassword = false;
               else if ( (mqo.userID != null) && (mqo.password != null) )
                  setUserIDPassword = false;
            }
            else
            {
               Class se = getSEClass(mqo.securityExit, mqo.securityExitPath);
               mqht.put(MQC.SECURITY_EXIT_PROPERTY, (MQSecurityExit) se.newInstance());
            }
         }
         
         if (setUserIDPassword)
         {
            if (mqo.userID != null)
               mqht.put(MQC.USER_ID_PROPERTY, mqo.userID);

            if (mqo.password != null)
               mqht.put(MQC.PASSWORD_PROPERTY, mqo.password);
         }

         /*
          * SSL
          */
         if (mqo.cipherSuiteName != null)
            mqht.put(MQC.SSL_CIPHER_SUITE_PROPERTY, mqo.cipherSuiteName);

         if (mqo.distinguishedName != null)
            mqht.put(MQC.SSL_PEER_NAME_PROPERTY, mqo.distinguishedName);

         if (mqo.trustedStore != null)
            System.setProperty("javax.net.ssl.trustStore", mqo.trustedStore);

         if (mqo.trustedStorePasswd != null)
            System.setProperty("javax.net.ssl.trustStorePassword", mqo.trustedStorePasswd);

         if (mqo.keyStore != null)
            System.setProperty("javax.net.ssl.keyStore", mqo.keyStore);

         if (mqo.keyStorePasswd != null)
            System.setProperty("javax.net.ssl.keyStorePassword", mqo.keyStorePasswd);

         if ( ((mqo.ldapServer != null)) && (mqo.ldapServerPort != 0) )
         {
            LDAPCertStoreParameters ldapcsparms = new LDAPCertStoreParameters(mqo.ldapServer, mqo.ldapServerPort);
            CertStore cs = CertStore.getInstance("LDAP", ldapcsparms);
            ArrayList al = new ArrayList();
            al.add(cs);
            mqht.put(MQC.SSL_CERT_STORE_PROPERTY, al);
         }

         _qMgr = new MQQueueManager(mqo.qMgrName, mqht);
      }
      else if (mqo.CCDTFile != null)
      {
         String trustedStore = System.getenv("MQSSLKEYR");
         if (trustedStore != null)
            System.setProperty("javax.net.ssl.trustStore", trustedStore + ".jks");

         File file = new File(mqo.CCDTFile);
         URL url = file.toURL();
         
         if (mqo.userID != null)
         {
            Hashtable mqht = new Hashtable();
   
            if (mqo.userID != null)
               mqht.put(MQC.USER_ID_PROPERTY, mqo.userID);

            if (mqo.password != null)
               mqht.put(MQC.PASSWORD_PROPERTY, mqo.password);
            
            _qMgr = new MQQueueManager(mqo.qMgrName, mqht, url);
         }
         else
         {
            _qMgr = new MQQueueManager(mqo.qMgrName, url);
         }
      }
      else
      {
         _qMgr = new MQQueueManager(mqo.qMgrName);
      }
   }

   /**
    *
    * @param clazzName
    * @param pathToJar
    * @return
    */
   private Class getSEClass(String clazzName, String pathToJar) throws MalformedURLException, ClassNotFoundException
   {
      Class cls = null;

      // Create a File object on the root of the directory containing the class file
      File file = new File(pathToJar);

      // Convert File to a URL
      URL url = file.toURL();
      URL[] urls = new URL[]
      {
         url
      };

      // Create a new class loader with the directory
      ClassLoader cl = new URLClassLoader(urls);

      // Load in the class; MyClass.class should be located in
      // the directory file:/c:/myclasses/com/mycompany
      cls = cl.loadClass(clazzName);

      return cls;
   }

   /**
    * Disconnect from the queue manager
    */
   public void disconnect()
   {
      try
      {
         if (_qMgr != null)
            _qMgr.disconnect();
      }
      catch (MQException e)
      {}
   }

   /**
    * Open the input queue
    *
    * @param qObject
    * @throws MQException
    */
   public MQQueue openInQ(QueueObject qObject) throws MQException
   {
      MQQueue inQ = null;
      
      inQ = _qMgr.accessQueue( qObject.qName,
                               MQC.MQOO_INPUT_AS_Q_DEF + MQC.MQOO_FAIL_IF_QUIESCING,
                               null,
                               null,
                               null );

      return inQ;
   }
}
